<?php
class SSPdataTables
{
	public static function dataTables($columns,$primaryKey,$table)
	{
		try {
			$bindings = array();
			$limit = self::limit($_GET, $columns);
			$order = self::order($_GET, $columns);
			$where = self::filter($_GET, $columns, $bindings);
			/* SQL QUERIES */
			/* DISPLAY DATA */
			$sQuery = "SELECT  `".implode("`, `", self::pluck($columns, 'db'))."`		
					FROM   $table
					$where
					$order
					$limit ";
			$query=DB::query(Database::SELECT, $sQuery, FALSE)->parameters($bindings);
			$aResultFilterTotal=$query->execute()->as_array();
			//fb((string)$query);
			/* DATA LENGTH FILTERED */
			$sQuery = " SELECT COUNT( $primaryKey ) as aantal
						FROM   $table
						$where ";
			$iFilteredTotal=DB::query(Database::SELECT, $sQuery, FALSE)->parameters($bindings)->execute()->get('aantal');

			/* DATA LENGTH TOTAL */
			$sQuery = " SELECT COUNT( $primaryKey ) as aantal
						FROM $table ";
			$iTotal=DB::query(Database::SELECT, $sQuery, FALSE)->execute()->get('aantal');

	    	/* OUTPUT */
			$output = array(
				
				"recordsTotal"    => intval( $iTotal ),
				"recordsFiltered" => intval( $iFilteredTotal ),
				"data"            => self::data_output($columns, $aResultFilterTotal)
			);
		}catch(Exception $error){
			$output = array(
				"recordsTotal" => 0,
				"recordsFiltered" => 0,
				"data" => array(),
				"error" => $error->getMessage()
			);
			Log::instance()->add(Log::WARNING, $error->getMessage());
		}
		return $output;
	}
	
	/**
	 * Create the data output array for the DataTables rows
	 *
	 *  @param  array $columns Column information array
	 *  @param  array $data    Data from the SQL get
	 *  @return array          Formatted data in a row based format
	 */
	static function data_output ( $columns, $data )
	{
		$out = array();
		for ( $i=0, $ien=count($data) ; $i<$ien ; $i++ ) {
			$row = array();
			for ( $j=0, $jen=count($columns) ; $j<$jen ; $j++ ) {
				$column = $columns[$j];
				// Is there a formatter?
				if (isset($column['formatter']) && !empty($data[$i][ $column['db'] ])) {
					$row[ $column['dt'] ] = $column['formatter']( $data[$i][ $column['db'] ], $data[$i] );
				}
				else {
					$row[ $column['dt'] ] = $data[$i][ $columns[$j]['db'] ];
				}
			}
			$out[] = $row;
		}
		return $out;
	}
	
	/*LIMIT*/
	static function limit($request, $columns)
	{
		$limit = '';
		if ( isset($request['start']) && $request['length'] != -1 ) {
			$limit = "LIMIT ".intval($request['start']).", ".intval($request['length']);
		}
		return $limit;
	}
	
	/* ORDER */
	static function order($request, $columns)
	{
		$order = '';
		if ( isset($request['order']) && count($request['order']) ) {
			$orderBy = array();
			$dtColumns = self::pluck( $columns, 'dt' );
			for ( $i=0, $ien=count($request['order']) ; $i<$ien ; $i++ ) {
				// Convert the column index into the column data property
				$columnIdx = intval($request['order'][$i]['column']);
				$requestColumn = $request['columns'][$columnIdx];
				$columnIdx = array_search( $requestColumn['data'], $dtColumns );
				$column = $columns[ $columnIdx ];
				if ( $requestColumn['orderable'] == 'true' ) {
					$dir = $request['order'][$i]['dir'] === 'asc' ? 'ASC' : 'DESC';
					$orderBy[] = '`'.$column['db'].'` '.$dir;
				}
			}
			$order = 'ORDER BY '.implode(', ', $orderBy);
		}
		return $order;
	}

	/* FILTER */
	static function filter($request, $columns, &$bindings)
	{
		$globalSearch = array();
		$columnSearch = array();
		$dtColumns = self::pluck( $columns, 'dt' );
		if ( isset($request['search']) && $request['search']['value'] != '' ) {
			$str = $request['search']['value'];
			for ( $i=0, $ien=count($request['columns']) ; $i<$ien ; $i++ ) {
				$requestColumn = $request['columns'][$i];
				$columnIdx = array_search( $requestColumn['data'], $dtColumns );
				$column = $columns[ $columnIdx ];
				if ($requestColumn['searchable'] == 'true') {
					if($request['search']['regex'] == 'true'){
						$binding = self::bind( $bindings, $str );
						$globalSearch[] = "`".$column['db']."` REGEXP ".$binding;
					} else if(strpos($str, "#µ=") === 0){
						$binding = self::bind( $bindings, substr($str,4));
						$globalSearch[] = "`".$column['db']."` = ".$binding;
					} else {
						$binding = self::bind( $bindings, '%'.$str.'%' );
						$globalSearch[] = "`".$column['db']."` LIKE ".$binding;
					}
					
				}
			}
		}

		// Individual column filtering
		for ( $i=0, $ien=count($request['columns']) ; $i<$ien ; $i++ ) {
			$requestColumn = $request['columns'][$i];
			$columnIdx = array_search( $requestColumn['data'], $dtColumns );
			$column = $columns[ $columnIdx ];
			$str = $requestColumn['search']['value'];
			if ( $requestColumn['searchable'] == 'true' && $str != '' ) {
				if($requestColumn['search']['regex'] == 'true'){
					$binding = self::bind( $bindings, $str );
					$columnSearch[] = "`".$column['db']."` REGEXP ".$binding;
				} else if(strpos($str, "#µ=") === 0){
					$binding = self::bind( $bindings, substr($str,4));
					$columnSearch[] = "`".$column['db']."` = ".$binding;
				}else {
					$binding = self::bind( $bindings, '%'.$str.'%');
					$columnSearch[] = "`".$column['db']."` LIKE ".$binding;
				}
				
			}
		}
		
		// Combine the filters into a single string
		$where = '';
		if ( count( $globalSearch ) ) {
			$where = '('.implode(' OR ', $globalSearch).')';
		}
		if ( count( $columnSearch ) ) {
			$where = $where === '' ?
				implode(' AND ', $columnSearch) :
				$where .' AND '. implode(' AND ', $columnSearch);
		}
		if ( $where !== '' ) {
			$where = 'WHERE '.$where;
		}
		return $where;
	}

	/* bind parameters */
	static function bind ( &$a, $val )
	{
		$key = ':binding_'.count( $a );
		$a[$key] = $val;
		return $key;
	}
	
	/* GET PROPERTY FROM ARRAY */
	static function pluck ( $a, $prop )
	{
		$out = array();
		for ( $i=0, $len=count($a) ; $i<$len ; $i++ ) {
			$out[] = $a[$i][$prop];
		}
		return $out;
	}


}

?>